!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              DEFINE_TR_CLOUD    Jan. 2014                                    |
!                                                                              |
!              based on define_cloud                                           |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine define_trcloud (trcl,surfd,params) !might use threadinfo

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! if irestart=0, this routine allocates and creates the cloud of tracking
! particles present in the system. Otherwise it reads from a user supplied file name the surfaces 
! as they were at the end of a previous run. In this case, since the run output 
! files contain all the octree+lsf+icloud+surface informations, the routine 
! first reads dummy parameters until it gets to the real interesting 
! cloud information 

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi
!use threads

implicit none

include 'mpif.h'

type (tracking_cloud) trcl
type (tracking_surfaced) surfd
type (parameters) params

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
logical isostasyout,isobcout,nestout,compactionout,ptrackingout
integer noctree,nnode,nleaves,nface,nlsf,clnp,istep_prev
double precision current_time

integer err,iproc,nproc,ierr,ilsf,i,j

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)


!------------------------------------------------------------------------------|
! Allocating the tracking and surfaced cloud and filling with default values   |
!------------------------------------------------------------------------------|

!allocate tracking cloud
trcl%np=0
allocate (trcl%x(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%x in define_trcloud$')
allocate (trcl%y(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%y in define_trcloud$')
allocate (trcl%z(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%z in define_trcloud$')
allocate (trcl%x0(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%x0 in define_trcloud$')
allocate (trcl%y0(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%y0 in define_trcloud$')
allocate (trcl%z0(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%z0 in define_trcloud$')
allocate (trcl%lsf0(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%lsf0 in define_trcloud$')
allocate (trcl%dist(params%ntrack), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%dist in define_trcloud$')
allocate (trcl%xhist(params%ntrack,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%xhist in define_trcloud$')
allocate (trcl%yhist(params%ntrack,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%yhist in define_trcloud$')
allocate (trcl%zhist(params%ntrack,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%zhist in define_trcloud$')
allocate (trcl%temphist(params%ntrack,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%temphist in define_trcloud$')
allocate (trcl%presshist(params%ntrack,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%presshist in define_trcloud$')

!allocate surfaced particles
surfd%np=0
allocate (surfd%x(1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%x in define_tr_cloud$')
allocate (surfd%y(1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%y in define_tr_cloud$')
allocate (surfd%z(1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%z in define_tr_cloud$')
allocate (surfd%xhist(1,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%xhist in define_tr_cloud$')
allocate (surfd%yhist(1,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%yhist in define_tr_cloud$')
allocate (surfd%zhist(1,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%zhist in define_tr_cloud$')
allocate (surfd%temphist(1,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%temphist in define_tr_cloud$')
allocate (surfd%presshist(1,params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%presshist in define_tr_cloud$')
allocate (surfd%time(params%nstep-params%tr_activation+1), stat=err) ; if (err.ne.0) call stop_run ('Error alloc surfd%time in define_tr_cloud$')

!filling arrays with default values
trcl%x = -1.d0
trcl%y = -1.d0
trcl%z = -1.d0
trcl%x0 = -1.d0
trcl%y0 = -1.d0
trcl%z0 = -1.d0
trcl%lsf0 = -1.d0 !In what range do lsf's lie? Use better default value?
trcl%dist = 0.d0
trcl%xhist = -1.0
trcl%yhist = -1.0
trcl%zhist = -1.0
trcl%temphist = -1.0
trcl%presshist = 0.0

!allocate surfaced particles
surfd%x = -1.0
surfd%y = -1.0
surfd%z = -1.0
surfd%xhist = -1.0
surfd%yhist = -1.0
surfd%zhist = -1.0
surfd%temphist = -1.0
surfd%presshist = 0.0
surfd%time = -1.0

!------------------------------------------------------------------------------|
! Reading from restart file                                                    |
!------------------------------------------------------------------------------|

!If restart file has particle tracking information, read and load
!This porcedure reads dummy parameters until it comes to the relevant information
!at the end of the file.
load: if (params%irestart > 0 .AND. params%irestart>params%tr_activation) then
   open (19,file=trim(params%restartfile),status='old',form='unformatted')
   ! Read version number
   read (19) 

   ! Read optional output flags
   read (19) isostasyout,isobcout,nestout,compactionout,ptrackingout

   if (.not. ptrackingout) then
       call stop_run ('Error: No tracking cloud information in restart file to read from!')
   end if

   !Read array lengths
   read (19) noctree,nnode,nleaves,nface,nlsf,clnp,current_time 

   ! Dummy reads
   ! info on octree solve nodes
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   ! Isostasy and compaction output
   if (isostasyout) read (19)
   if (compactionout) read (19)
   !icon array
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   !compaction output
   if (compactionout) read (19)
   ! octree information
   read (19)
   ! bad face information
   read (19)
   ! void information
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   ! bad faces information
   read (19)
   ! surface information
   do ilsf=1,nlsf                                  
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
      read (19)
   enddo
   
   !Cloud Information
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)
   read (19)

   ! read isostasy basal displacement array (optional) - dwhipp 07.13
   if (isobcout) then
     read (19)
     read (19)
   endif

   ! read nest info (optional) - dwhipp 07.13
   if (nestout) read(19)

   ! read compaction density strings (optional) - dwhipp 07.13
   if (compactionout) then
     read (19)
     do j=1,2**params%levelmax_oct
       do i=1,2**params%levelmax_oct
         read (19)
         read (19) 
       enddo
     enddo
   endif

   !Reading tracking particle information
   readtrcl: if (ptrackingout) then
       read (19) trcl%np, surfd%np, istep_prev
       !number of cloud particles have to agree
       if (trcl%np /= params%ntrack) then
         if (iproc==0) write (*,'(a)') 'Error: Number of tracking paricles in input and restart file do not agree!'
         if (iproc==0) write (*,'(a)') 'This is not supported and DOUAR cannot continue.'
         call stop_run ('Error: Different number of tracing particles on restart in define_cloud$')
       endif
       !Restart timestep has to be correct in input file
       if (istep_prev /= params%irestart) then
         if (iproc==0) write (*,'(a)') 'Error: irestart in input file does not agree with timesteps elapsed in'
         if (iproc==0) write (*,'(a)') 'restart file!'
         call stop_run ('Error: irestart is not the number of elapsed timesteps in restart file$')
       endif
       
       !read tracking cloud up to current time step
       read(19) trcl%x
       read(19) trcl%y
       read(19) trcl%z
       read(19) trcl%x0
       read(19) trcl%y0
       read(19) trcl%z0
       ! correct for vertical exaggeration
       trcl%z=trcl%z/params%vex
       trcl%z0=trcl%z0/params%vex
       read(19) trcl%lsf0
       read(19) trcl%dist 
       do i = 1,istep_prev
           read(19) trcl%xhist(:,i)
           read(19) trcl%yhist(:,i)
           read(19) trcl%zhist(:,i)
           ! correct for vertical exaggeration
           trcl%zhist(:,i)=trcl%zhist(:,i)/params%vex
           read(19) trcl%temphist(:,i)
           read(19) trcl%presshist(:,i)
       end do

       !read surfaced cloud up to current time step
       read(19) surfd%x
       read(19) surfd%y
       read(19) surfd%z
       ! correct for vertical exaggeration
       surfd%z=surfd%z/params%vex
       read(19) surfd%time(1:istep_prev)
       do i = 1,istep_prev
           read(19) surfd%xhist(:,i)
           read(19) surfd%yhist(:,i)
           read(19) surfd%zhist(:,i)
           ! correct for vertical exaggeration
           surfd%zhist(:,i)=surfd%zhist(:,i)/params%vex
           read(19) surfd%temphist(:,i)
           read(19) surfd%presshist(:,i)
       end do

   end if readtrcl

   close (19)

end if load 

return
end

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
