!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              UPDATE_TRCLOUD_STRUCTURE    Jan. 2015                           |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine update_trcloud_structure(trcl,surfd,ov,osolve,params,istep) !might use threadinfo

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! this file contains the operations done to 'groom' the tracking cloud - check 
! the position of particles, save surfaced ones, discard others and inject new 
! ones.
! In the first run, the cloud is created.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|
      
use definitions
!use mpi

implicit none

include 'mpif.h'

type (tracking_cloud) trcl
type (tracking_surfaced) surfd
type (octreev) ov
type (octreesolve) osolve
type (parameters) params
integer istep

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer iproc,nproc,ierr
integer i,j,k,tgridshape,gridstep
integer trstep,ntrackx,ntracky,ntrackz
integer,dimension(:),allocatable::surfd,sided,surfd_pos,sided_pos

double precision eps,dx,dy,dz
double precision,dimension(:),allocatable::lsf0,temp,press,lsf

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

!time steps since tracking activation
trstep = istep-params%tr_activation+1
ntrackx = params%ntrackx
ntracky = params%ntracky
ntrackz = params%ntrackz


!------------------------------------------------------------------------------|
! If cloud is empty, this is activation step: Create Cloud
!------------------------------------------------------------------------------|
if (trcl%np == 0) then
    if (params%gridtrack) then
    !Regularly gridded tracking cloud
        trgridshape = mod(params%ntrackx,2)+mod(params%ntracky,2)+mod(params%ntrackz,2)
        if (trgridshape==0) then
        !simple cubic grid
            dx = (params%trboxxmax-params%trboxxmin)/ntrackx
            dy = (params%trboxymax-params%trboxymin)/ntracky
            dz = (params%trboxzmax-params%trboxzmin)/ntrackz

            !create tracking particles
            gridstep = 1
            do i = 1,ntrackx
                do j = 1,ntracky
                    do k = 1,ntrackz
                        trcl%x(gridstep) = params%trboxxmin+(i-0.5d0)*dx
                        trcl%y(gridstep) = params%trboxymin(j-0.5d0)*dy
                        trcl%z(gridstep) = params%trboxzmin(k-0.5d0)*dz
                        gridstep = gridstep+1
                    end do
                end do
            end do

        else
        !tgridshape=3, bcc-type gric
            dx = 2.d0*(params%trboxxmax-params%trboxxmin)/(ntrackx+1)
            dy = 2.d0*(params%trboxxmax-params%trboxxmin)/(ntracky+1)
            dz = 2.d0*(params%trboxxmax-params%trboxxmin)/(ntrackz+1)

            !create tracking particles
            gridstep = 1
            do i = 1,(ntrackx+1)/2
                do j = 1,(ntracky+1)/2
                    do k = 1,(ntrackz+1)/2
                        trcl%x(gridstep) = params%trboxxmin+(i-0.5d0)*dx
                        trcl%y(gridstep) = params%trboxymin(j-0.5d0)*dy
                        trcl%z(gridstep) = params%trboxzmin(k-0.5d0)*dz
                        gridstep = gridstep+1
                    end do
                end do
            end do
            do i = 1,(ntrackx-1)/2
                do j = 1,(ntracky-1)/2
                    do k = 1,(ntrackz-1)/2
                        trcl%x(gridstep) = params%trboxxmin+i*dx
                        trcl%y(gridstep) = params%trboxymin+j*dy
                        trcl%z(gridstep) = params%trboxzmin+k*dz
                        gridstep = gridstep+1
                    end do
                end do
            end do
        end if

        if (gridstep-1 == params%ntrack) then
            trcl%np = params%ntrack
        else
            call stop_run ('error in tracking cloud grid injection')
        end if
        
    else
    !Randomly distributed tracking cloud
        call random_number (trcl%x)
        call random_number (trcl%y)
        call random_number (trcl%z)
        do i = 1,params%ntrack
            trcl%x(i) = params%trboxxmin+(params%trboxxmax-params%trboxxmin)*trcl%x(i)
            trcl%y(i) = params%trboxymin+(params%trboxymax-params%trboxymin)*trcl%y(i)
            trcl%z(i) = params%trboxzmin+(params%trboxzmax-params%trboxzmin)*trcl%z(i)
        end do
        trcl%np = params%ntrack
    end if
end if


!------------------------------------------------------------------------------|
! Interpolate values from osolve onto tracking particles                       |
! -- taken form update_cloud_fields.f90                                        |
!------------------------------------------------------------------------------|

!Get upper surface level set function
allocate (lsf(ov%nnode),stat=err) ; if (err.ne.0) call stop_run ('Error alloc lsf in update_cloud_fields$')
lsf=0.d0
if (osolve%nlsf.ne.0) lsf=osolve%lsf(1:osolve%nnode,1)

!allocate temporary arrays
allocate (lsf0(trcl%np),stat=err) ; if (err.ne.0) call stop_run ('Error alloc lsf0 in update_trcloud$')
allocate (temp(trcl%np),stat=err) ; if (err.ne.0) call stop_run ('Error alloc temp in update_trcloud$')
allocate (press(trcl%np),stat=err) ; if (err.ne.0) call stop_run ('Error alloc press in update_trcloud$')

lsf0=0.d0
temp=0.d0
press=0.d0

!interpolate values
do k=1+iproc,trcl%np,nproc
   call octree_interpolate_three (3,ov%octree,ov%noctree,ov%icon, &
                                 ov%nleaves,ov%nnode, &
                                 trcl%x(k),trcl%y(k),trcl%z(k), &
                                 lsf,lsf0(k), &
                                 ov%temp,temp(k), &
                                 ov%spressure,press(k)) !Using smoothed pressure - okay to do so?
enddo

call mpi_allreduce (lsf0, trcl%lsf0, trcl%np, mpi_double_precision,mpi_sum,mpi_comm_world,ierr)
call mpi_allreduce (real(temp), trcl%temp(:,trstep), trcl%np, mpi_real,mpi_sum,mpi_comm_world,ierr)
call mpi_allreduce (real(press),trcl%press(:,trstep),trcl%np, mpi_real,mpi_sum,mpi_comm_world,ierr)

deallocate (lsf)
deallocate (lsf0)
deallocate (temp)
deallocate (press)

!------------------------------------------------------------------------------|
! Surfaced particles (lsf0>0) are saved and reset                              |
!------------------------------------------------------------------------------|


!find number and index of surfaced particles
allocate (surfd(trcl%np),stat=err) ; if (err.ne.0) call stop_run ('Error alloc surf in update_trcloud$')
surfd = 1
nsurfd = 0

do i = 1,trcl%np
    if (trcl%lsf0(i) > 0) then
        surfd(i) = 1
        nsurfd = nsurfd+1
end do

!------------------------------------------------------------------------------|
! Move tracking particles                                                      |
!------------------------------------------------------------------------------|

!------------------------------------------------------------------------------|
! Exited particles are discarded and reset                                     |
!------------------------------------------------------------------------------|

return
end
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
